@@ -5,18 +5,20 @@ Written by Arnt Gulbrandsen <agulbra@troll.no> and copyright 1995
 Troll Tech AS, Postboks 6133 Etterstad, 0602 Oslo, Norway, fax +47
 22646949.
 Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>
-and Randolf Skerka <rskerka@metronet.de>.
+and Randolf Skerka <Randolf.Skerka@gmx.de>.
 Copyright of the modifications 1997.
 Modified by Kent Robotti <robotti@erols.com>. Copyright of the
 modifications 1998.
 Modified by Markus Enzenberger <enz@cip.physik.uni-muenchen.de>.
 Copyright of the modifications 1998.
 Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>.
-Copyright of the modifications 1998.
+Copyright of the modifications 1998, 1999.
 
 See file COPYING for restrictions on the use of this software.
 */
 
+#include "leafnode.h"
+
 #include <fcntl.h>
 #include <sys/uio.h>
 #include <sys/param.h>
@@ -34,17 +36,10 @@ See file COPYING for restrictions on the use of this software.
 #include <unistd.h>
 #include <dirent.h>
 
-#include "leafnode.h"
-
 #define   STRNCMPEQ(s1, s2, n)    (*(s1) == *(s2) && strncmp((s1), (s2), n) == 0)
 
-extern int errno;
-extern int h_errno;
 extern struct state _res;
 
-struct newsgroup * active;
-struct newsgroup * blocked;
-
 /* insert or update a newsgroup in the global list */
 void insertgroup(const char * name, int first, int last, const char * desc,
 		 int age ) {
@@ -64,37 +59,50 @@ void insertgroup(const char * name, int first, int last, const char * desc,
 		a = &((*a)->right);
 	    else {
 	    	/* update newsgroup description, nothing else */
-		if (desc && *desc) {
-		    /* This leaks memory if the new desc > old one, but I
-		       cannot see a way round this... */
-		    if (strlen((*a)->desc) >= strlen(desc))
-			strcpy((*a)->desc, desc);
-		    else
-			(*a)->desc = strdup( desc );
+		if ( (*a)->desc ) {
+		    if ( !desc )
+			return;
+		    if ( ( strcmp( desc, "-x-") == 0 ) ||
+			 ( strcmp( desc, "local group" ) == 0 ) )
+		        return;
+		    free( (*a)->desc );
 		}
+		(*a)->desc = strdup( desc );
 		return;
 	    }
 	} else {
 	    /* create new newsgroup entry */
 	    *a = (struct newsgroup *)
-		 critmalloc(sizeof(struct newsgroup) + 2 +
-			    strlen( name ) + strlen( desc ),
+		 critmalloc(sizeof(struct newsgroup),
 			    "Building newsgroups info list");
 	    if (!*a)
 		return;
 	    (*a)->left = (*a)->right = NULL;
-	    (*a)->first = first;
-	    (*a)->last = last;
+	    (*a)->first = (first) ? first : 1;
+	    (*a)->last = (last>=first) ? last : 1;
 	    (*a)->age = age;
-	    (*a)->name = (char*)(1+*a);
-	    strcpy( (*a)->name, name);
-	    (*a)->desc = strlen((*a)->name)+1+((*a)->name);
-	    strcpy( (*a)->desc, desc );
+	    (*a)->name = strdup( name );
+	    if ( (*a)->name == NULL ) {
+		syslog( LOG_ERR, "Not sufficient memory for newsgroup %s",
+			name );
+		free(*a);
+		return;
+	    }
+	    (*a)->desc = strdup( desc );
 	    return;
 	}
     }
 }
 
+/*
+ * check whether group is represented in interesting.groups
+ */
+int isinteresting( char * groupname ) {
+    struct stat st;
+
+    sprintf( s, "%s/interesting.groups/%s", spooldir, groupname );
+    return(stat( s, &st ) + 1);
+}
 
 /* find a group by name */
 struct newsgroup * findgroup(const char* name) {
@@ -115,26 +123,6 @@ struct newsgroup * findgroup(const char* name) {
 }
 
 
-/* find a blocked group by name */
-/* from leafnode-1.5(enz)
-int findblocked(const char* name) {
-    struct newsgroup * a;
-    int c;
-
-    a = blocked;
-    while ( a ) {
-        c = strcmp( a->name, name );
-        if ( c < 0 )
-            a = a->left;
-        else if ( c > 0 )
-            a = a->right;
-        else
-            return TRUE;
-    }
-    return FALSE;
-}
-*/
-
 static void freeactive( struct newsgroup * g ) {
     if ( g ) {
 	freeactive( g->right );
@@ -167,97 +155,135 @@ static void freeactive( struct newsgroup * g ) {
    care, a _perfectly_ balanced tree */
 
 void readactive( void ) {
-    char * p;
-    char * q;
+    char * p, *q, *r, *lastline, *desc;
     int first, last;
     time_t age;
-    int fd;
+    FILE * f;
     struct newsgroup * g;
     struct stat st;
 
     int line;
     struct newsgroup *levels[32]; /* theoretically finite size :) */
-    int l;
+    int l, n, error;
 
-    static char * stuff;
+    char * stuff;
 
-    if ( stuff ) {
-	freeactive( active );
-	active = NULL;
-	free( (char*)stuff );
-	stuff = NULL;
-    }
+    freeactive( active );
+    active = NULL;
 
     strcpy(s, spooldir);
     strcat(s, "/leaf.node/groupinfo");
-    fd = open( s, O_RDONLY );
     if ( stat( s, &st ) ) {
 	syslog( LOG_ERR, "can't stat %s: %m", s );
 	return;
-    }
-    stuff = critmalloc( st.st_size+1, "Reading group info" );
-    if ( (fd=open( s, O_RDONLY))<0 ||
-	 (read( fd, stuff, st.st_size ) < st.st_size) ) {
-	syslog( LOG_ERR, "can't open/read %s: %m", s );
+    } else if ( !S_ISREG( st.st_mode ) ) {
+	syslog( LOG_ERR, "%s not a regular file" );
 	return;
-    } else {
-	close( fd );
-	stuff[st.st_size] = '\0'; /* 0-terminate string */
     }
+    stuff = critmalloc( st.st_size+2, "Reading group info" );
+    if (( f = fopen( s, "r" )) != NULL ) {
+	n = fread( stuff, 1, st.st_size, f );
+	if ( n < st.st_size )
+	    syslog( LOG_ERR,
+		    "Groupinfo file truncated while reading: %d < %d .",
+		    n, st.st_size );
+	fclose( f );
+    }
+    else {
+	syslog( LOG_ERR, "unable to open %s: %m", s );
+	return;
+    }
+
+    n = ( n > st.st_size ) ? st.st_size : n ;
+	/* to read truncated groupinfo files correctly */
+    stuff[n] = '\n';
+    stuff[n+1] = '\0';	/* 0-terminate string */
+
+    /* delete spurious 0-bytes */
+    while (( p = memchr( stuff, '\0', st.st_size ) ) != NULL )
+        *p = ' ';       /* \n might be better, but produces more errors */
 
     line = 1;
     for( l=0; l<32; levels[l++] = NULL )
 	;
 
     p = stuff;
-    while ( p && *p ) {
-	q = p;
-	p = strchr(p, ' ');
-	if (p && *p)
-	    *p++ = '\0';
-	else {
-	    syslog( LOG_ERR, "groupinfo file may be corrupted" );
-	    *q = '\0' ;		/* skip defective group */
-	}
-	if ( *q ) {
-	    last = p ? strtol( p, &p, 10 ) : 0;
-	    first = p ? strtol( p, &p, 10 ) : 1;
-	    age = p ? strtol( p, &p, 10 ) : 1;
-	    while (p && *p && isspace(*p))
-		p++;
-	    if (first < 1)
-		first = 1;
+    error = 0;
+    while ( (p < (stuff+n)) && (( q = strchr( p, '\n' )) != NULL ) ) {
+	*q = '\0' ;
+	lastline = p;
+        if (( ( r = strchr( p, ' ' )) != NULL ) && isalnum((int)*p) ) {
+            *r = '\0' ;
+            p = r + 1 ;
+        } else
+            error = 1;
+        if ( !error ) {
+            last = *p ? strtol( p, &r, 10 ) : 1;
+            if ( r == p )
+                error = 1;
+            else {
+                p = r;
+                first = *p ? strtol( p, &r, 10 ) : 1;
+                if ( r == p )
+                    error = 1;
+                else {
+                    p = r;
+                    age = *p ? strtol( p, &r, 10 ) : 0;
+                    if ( r == p )
+                        error = 1;
+		    else
+			p = r;
+                }
+            }
+        }
 
+        if ( !error ) {
+            while (p && *p && isspace((int)*p))
+                p++;
+            if ( p && *p )
+                desc = strdup( p );
+            else
+                error = 1;
+        }
+
+        if ( error ) {
+            syslog( LOG_ERR,
+		    "Groupinfo file possibly truncated or damaged: %s\n",
+		    lastline);
+            error = 0;
+        } else {
 	    g = (struct newsgroup *) critmalloc(sizeof(struct newsgroup),
-						"reading groupinfo");
+						"reading groupinfo" );
 	    g->right = g->left = NULL;
-	    g->first = first;
-	    g->last = last;
+	    g->first = (first) ? first : 1;
+	    g->last = (last>=first) ? last : 1 ;
 	    g->age = age;
-	    g->name = q;
-	    g->desc = p;
+	    g->name = strdup(lastline);
+	    g->desc = desc;
 
-	    for( l=0; (line&(1<<l))==0; l++ )
-		;
-	    if ( l ) {
-		g->right = levels[l-1];
-	    }
-	    if ( active == g->right )
-		active = g;
-	    if ( l<31 && levels[l+1] && !levels[l+1]->left )
-		levels[l+1]->left = g;
-	    levels[l] = g;
+            /* Inserting newsgroup into tree */
+            for( l=0; (line&(1<<l))==0; l++ )
+                ;
+            if ( l ) {
+                g->right = levels[l-1];
+            }
+            if ( active == g->right )
+                active = g;
+            if ( l<31 && levels[l+1] && !levels[l+1]->left )
+                levels[l+1]->left = g;
+            levels[l] = g;
 
-	    p = strchr( p, '\n' );
-	    if ( p )
-		*p++ = '\0';
-	    line++;
-	} else {
-	    p = strchr( p, '\n' );
-	    if ( p )
-		p++;
+            p = strchr( p, '\n' );
+            if ( p )
+                *p++ = '\0';
+            line++;
 	}
+
+        p = q + 1;
     }
+    free( stuff );
+
+    /* More tree stuff */
     g = NULL;
     for ( l=0; l<31; l++ ) {
 	if ( levels[l] ) {
@@ -277,108 +303,6 @@ void readactive( void ) {
     }
 }
 
-/* from leafnode-1.5(enz). This is more-or-less copied from readactive()
-void readblocked( void ) {
-    char * p;
-    char * q;
-    int fd;
-    struct newsgroup * g;
-    struct stat st;
-
-    int line;
-    struct newsgroup *levels[32];
-    int l;
-    int skiptoeol;
-
-    static char * stuff;
-
-    if ( stuff ) {
-        freeactive( blocked );
-        free( (char*)stuff );
-        stuff = NULL;
-    }
-    blocked = NULL;
-
-    strcpy(s, spooldir);
-    strcat(s, "/leaf.node/blocked");
-    fd = open( s, O_RDONLY );
-    if ( stat( s, &st ) ) {
-        return;
-    }
-    stuff = critmalloc( st.st_size+1, "Reading blocked group info" );
-    if ( (fd=open( s, O_RDONLY))<0 ||
-         (read( fd, stuff, st.st_size ) < st.st_size) ) {
-        syslog( LOG_ERR, "can't open/read %s: %m", s );
-        return;
-    } else {
-        close( fd );
-        stuff[st.st_size] = '\0';
-    }
-
-    line = 1;
-    for( l=0; l<32; levels[l++] = NULL )
-        ;
-
-    p = stuff;
-    while ( p && *p ) {
-        q = p;
-        while (!isspace(*p)) { p++; }
-        skiptoeol = (*p != '\n');
-        *p++ = '\0';
-        if ( *q ) {
-            g = (struct newsgroup *) critmalloc(sizeof(struct newsgroup),
-                                                "reading blocked");
-            g->right = g->left = NULL;
-            g->first = 0;
-            g->last = 0;
-            g->name = q;
-            g->desc = NULL;
-
-            for( l=0; (line&(1<<l))==0; l++ )
-                ;
-            if ( l ) {
-                g->right = levels[l-1];
-            }
-            if ( blocked == g->right )
-                blocked = g;
-            if ( l<31 && levels[l+1] && !levels[l+1]->left )
-                levels[l+1]->left = g;
-            levels[l] = g;
-
-            if (skiptoeol) {
-                p = strchr( p, '\n' );
-                if ( p )
-                    *p++ = '\0';
-            }
-            line++;
-        } else {
-            if (skiptoeol) {
-                p = strchr( p, '\n' );
-                if ( p )
-                    *p++ = '\0';
-            }
-        }
-    }
-    g = NULL;
-    for ( l=0; l<31; l++ ) {
-        if ( levels[l] ) {
-            if ( g && levels[l] &&
-                 levels[l]->left == NULL && levels[l]->right != g ) {
-                levels[l]->left = g;
-                g = NULL;
-            }
-            if ( !levels[l+1] ||
-                 ( levels[l] != levels[l+1]->left &&
-                   levels[l] != levels[l+1]->right ) ) {
-                if ( g )
-                    syslog( LOG_ERR, "2+5=2" );
-                g = levels[l];
-            }
-        }
-    }
-}
-*/
-
 /*
  * returns 1 if something could be written, 0 otherwise
  */
@@ -445,18 +369,18 @@ void fakeactive( void ) {
 	return;
     }
     while ( (de = readdir(d)) ) {
-	if ( isalnum( *(de->d_name) ) ) {
+	if ( isalnum((int)*(de->d_name) ) && chdirgroup(de->d_name, FALSE) ) {
 	    /* get first and last article from the directory. This is
 	     * the most secure way to get to that information since the
 	     * .overview files may be corrupt as well
+	     * If the group doesn't exist, just ignore the active entry.
 	     */
-	    chdirgroup( de->d_name );
 	    first = INT_MAX;
 	    last = 0;
 
 	    ng = opendir( "." );
 	    while ( ( nga = readdir( ng ) ) != NULL ) {
-		if ( isdigit ( *(nga->d_name ) ) ) {
+		if ( isdigit ((int) *(nga->d_name ) ) ) {
 		    i = strtol( nga->d_name, NULL, 10 );
 		    if ( i < first )
 			first = i;
@@ -465,12 +389,13 @@ void fakeactive( void ) {
 		}
 	    }
 	    if ( first > last ) {
-		first = 0;
+		first = 1;
 		last = 1;
 	    }
 	    closedir( ng );
-	    syslog( LOG_DEBUG, "parsed directory %s: first %d, last %d",
-		    de->d_name, first, last );
+	    if ( debugmode )
+	        syslog( LOG_DEBUG, "parsed directory %s: first %d, last %d",
+			de->d_name, first, last );
 	    insertgroup( de->d_name, first, last, NULL, 0 );
 	}
     }
