@@ -3,34 +3,30 @@
  * Input file : a checkgroups script that contains the name of
  *              the newsgroup and its description in one line
  *
- * Cornelius Krasel, April 1997
+ * Written and copyrighted by Cornelius Krasel, April 1997
  * Source code borrows a lot from fetch(1).
+ *
+ * See file COPYING for restrictions on the use of this software.
  */
 
+#include "leafnode.h"
 #include <sys/types.h>
-#ifdef BSD
-#include <sys/errno.h>
-#endif
 #include <ctype.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <stdlib.h>
-#include <pwd.h>
 #include <setjmp.h>
 #include <signal.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <syslog.h>
 #include <time.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <unistd.h>
 
-const int debug = 0;
-const int verbose = 0;
-
-#include "leafnode.h"
+int debug = 0;
+int verbose = 0;
 
 void process_input(char *s);
 
@@ -49,8 +45,8 @@ void process_input(char *s) {
         struct newsgroup * g;
 
         p = l;
-        if (isalnum(*p)) {
-            while (!isspace(*p))
+        if (isalnum((int)*p)) {
+            while (!isspace((int)*p))
                 p++;
             if (*p)
                 *p++ = '\0';
@@ -66,32 +62,24 @@ void process_input(char *s) {
 }
 
 int main(int argc, char *argv[] ) {
-    struct passwd * pw;
     FILE * f;
 
-    pw = getpwnam( "news" );
-    if ( !pw ) {
-        fprintf( stderr, "no such user: news\n" );
-        exit( 1 );
-    }
-
-    setgid( pw->pw_gid );
-    setreuid( pw->pw_uid, pw->pw_uid );
-
-    if ( getuid() != pw->pw_uid || getgid() != pw->pw_gid ) {
-        fprintf( stderr, "%s: must be run as news or root\n", argv[0] );
-        exit( 1 );
-    }
+    if ( ! initvars( argv[0] ) )
+	exit( 1 );
 
     if (argc != 2) {
-        fprintf( stderr, "Usage: %s [checkgrous file]\n", argv[0] );
+        fprintf( stderr, "Usage: %s [checkgroups file]\n", argv[0] );
         exit( 1 );
     }
 
     /* Check whether input file exists */
     if ( !( f = fopen( argv[1], "r" ))) {
-        fprintf( stderr, "%s: checkgroups file %s doesn't exist\n",
-	         argv[0], argv[1] );
+        if ( errno == EACCES )
+	    fprintf( stderr, "%s: not permitted to open %s\n",
+		     argv[0], argv[1] );
+	else
+	    fprintf( stderr, "%s: checkgroups file %s doesn't exist\n",
+		     argv[0], argv[1] );
         exit( 1 );
     }
     else
